package	jp.co.sds_corp.ratemixer.fx.filter;

import java.math.BigDecimal;
import java.util.Vector;

import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* 為替レートを履歴からの乖離率でフィルタリングします。
<PRE>
&lt;filter class="jp.co.sds_corp.ratemixer.fx.filter.SanityFilter"&gt;
	&lt;target ccy1="USD" ccy2="JPY" term="SP"/&gt;
	&lt;deviation&gt;30&lt;deviation&gt;
	&lt;times&gt;5&lt;times&gt;
&lt;/filter&gt;
</PRE>
*/
public class SanityFilter extends FxFilter
{
	private static final long serialVersionUID = 1L;
	BigDecimal	deviation;
	int	times;
	BigDecimal bDev;
	Vector<FxRate>	vecHist = new Vector<FxRate>();

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		deviation = new BigDecimal( config.get( "deviation" ))
					.divide( new BigDecimal( "100" ), BigDecimal.ROUND_DOWN );
		times = Integer.valueOf( config.get( "times" )).intValue();
	}

	/**
	* レートのフィルタリングを行なう
	* @param	ev		レート変更イベント
	*/
	public boolean isValid( ReceiveEvent ev ) throws Exception {
		if ( vecHist.size() == 0 ) {
			return true;
		}
		BigDecimal sumB = new BigDecimal( "0" );
		BigDecimal sumA = new BigDecimal( "0" );
		for ( int i = 0; i < vecHist.size(); i++ ) {
			FxRate hist = (FxRate )vecHist.get( i );
			sumB = sumB.add( hist.getBid() );
			sumA = sumA.add( hist.getAsk() );
		}
		FxRate rate = (FxRate )ev.getData();
		if ( !checkRate( sumB, rate.getBid() )) {
			return false;
		}
		if ( !checkRate( sumA, rate.getAsk() )) {
			return false;
		}
		addHist( rate );
		return true;
	}

	private boolean checkRate( BigDecimal sum, BigDecimal cur ) {
		BigDecimal avg = sum.divide( new BigDecimal( vecHist.size() ),
									BigDecimal.ROUND_DOWN );
		BigDecimal dev = cur.subtract( avg )
							.divide( avg, BigDecimal.ROUND_DOWN ).abs();
		if ( dev.compareTo( deviation ) > 0 ) {
			return false;
		}
		return true;
	}

	private void addHist( FxRate rate ) {
		vecHist.add( rate );
		if ( vecHist.size() > times ) {
			vecHist.remove( 0 );
		}
	}

	/**
	* 乖離(deviation)を設定します。
	* @param	var		設定するデータ
	*/
	public void setDeviation( BigDecimal var ) {
		this.deviation = var ;
	}
	/**
	* 乖離(deviation)を取得します。
	* @return	deviationの内容
	*/
	public BigDecimal getDeviation() {
		return deviation;
	}

	/**
	* 回数(times)を設定します。
	* @param	var		設定するデータ
	*/
	public void setTimes( int var ) {
		this.times = var ;
	}
	/**
	* 回数(times)を取得します。
	* @return	timesの内容
	*/
	public int getTimes() {
		return times;
	}

}
